(* A modified version of an example from page 135 of our textbook. *)

(* Here is a simple function using pattern matching. *)
fun f [] = 0
 |  f (a :: nil) = a
 |  f (_ :: b ::nil) = b
 |  f (_ :: _ :: c :: _) = c;


(* Here is the above function rewritten using a case expression.
   All of the pattern matching is now in the case expression.
   Notice how the syntax changed slightly when the patterns were
   moved from the body of a function to the body of a case expression. *)
fun g x =
  case x of
     [] => 0
   |  a :: nil => a
   |  _ :: b ::nil => b
   |  _ :: _ :: c :: _ => c;


(* This is the same function, just formated in a slightly different way. *)
fun g x =
  case x of
    [] => 0 |
    a :: nil => a |
    _ :: b :: nil => b |
    _ :: _ :: c :: _ => c;


(* This is how the book wrote the pattern matching on page 135. *)
(* The clauses in these patterns are NOT mutually exclusive. *)
(* It is possible that they are written this way to be more efficient. *)
fun h x =
  case x of
    _ :: _ :: c :: _ => c |  (* Match the most likely case first? *)
    _ :: b :: _ => b |       (* Use _ instead of nil for efficiency? *)
    a :: _ => a |
    nil => 0;