(* Compare the type signature of this function, *)
fun f (x, y) = x + y*y;

(* with this function's type signature. *)
fun g x y = x + y*y;

(* Look at the type signature of the following function. *)
fun h x = fn y => x + y*y;

(* As far as ML knows, the functions g and h are exactly the same!
   They are the curried version of f.
   Actually, h is the "curried version of f".
   In the case of g, ML has a syntax that automatically creates
   a "curried function". So g is a shorhand notation for h. *)


(* look at the type signature of the following values *)
g 5;
h 5;

val g5 = g 5; (* What is the "formula" for g5? *)
g5 5;
g5 2;

val h2 = h 2; (* What is the "formula" for h2? *)
h2 3;


(* Here is a curried function that also uses pattern matching. *)
fun incList x nil = nil
 |  incList x (y::ys) = x+y::incList x ys;


val i3 = incList 3;

i3 [3, 4, 5, 6, 7];


(* Here is another example. *)
fun chooser x nil = nil
 |  chooser x (y::ys) = (x<y)::chooser x ys;

 val ch0 = chooser 0;

 ch0 [~3, 0, 5, ~3, 6, ~1, 2];

 chooser ~2 [~3, 0, 5, ~3, 6, ~1, 2];